Imports System
Imports System.DirectoryServices
Imports System.Text
Imports System.IO
Imports System.Configuration
Imports System.Threading
Imports System.ServiceProcess
Imports System.Diagnostics
Imports System.Reflection

<Assembly: AssemblyTitle("IisLogHousekeeper 1.1 IIS6x edition")>
<Assembly: AssemblyDescription("IIS log housekeeping service")>
<Assembly: AssemblyCompany("www.808.dk")>
<Assembly: AssemblyProduct("IisLogHousekeeper")>
<Assembly: AssemblyCopyright("Creative Commons Attribution (CC BY 3.0) http://creativecommons.org/licenses/by/3.0/ Reference source http://www.808.dk/?code-iis-log-housekeeper-service")>
<Assembly: AssemblyVersion("1.1.0.0")>
<Assembly: CLSCompliant(True)> 

Public Class IisLogHousekeeper

  Private Shared ServiceThread As Thread
  Private Shared KeepRunning As Boolean = True

  Protected Overrides Sub OnStart(ByVal args() As String)
    WriteToEventLog("IisLogHousekeeper starting", 1110)
    ServiceThread = New Thread(AddressOf RunWorkerThread)
    ServiceThread.Start()
  End Sub

  Protected Overrides Sub OnStop()
    KeepRunning = False
    WriteToEventLog( _
      "IisLogHousekeeper stopping. Please note: the worker process " & _
      "will live on for up to 60 seconds before it terminates.", 1110)
  End Sub

  Public Sub RunWorkerThread()
    Dim aFolders() As String = {}
    If LCase(ConfigurationManager.AppSettings("UseManualLogLocations")) = "true" Then
      aFolders = Split(ConfigurationManager.AppSettings("ManualLogLocations"), "|")
    End If
    Dim sRunResult As String = ""
    Do While KeepRunning
      If ConfigurationManager.AppSettings("RunAt") = Date.Now.ToString("HH:mm") Then
        If LCase(ConfigurationManager.AppSettings("UseManualLogLocations")) = "false" Then
          Try
            aFolders = GetIisLogFoldersThroughADSI()
          Catch e As Exception
            WriteToEventLog("Could not get IIS log locations through ADSI. Error message: " & e.Message, 1112)
          End Try
        End If
        Try
          For i As Integer = 0 to UBound(aFolders)
            sRunResult = sRunResult & _
              DeleteLogFilesRecursive(aFolders(i), CInt(ConfigurationManager.AppSettings("LogRetentionDays")))
          Next
          If sRunResult <> "" Then
            WriteToEventLog(sRunResult, 1111)
            sRunResult = ""
          End If
        Catch e As Exception
          WriteToEventLog("An error occurred while cleaning up logs: " & e.Message, 1112)
        End Try
      End If
      Thread.Sleep(60000)
    Loop
  End Sub

  Public Function WriteToEventLog(ByVal Entry As String, _
     Optional ByVal EventID As Integer = 0, _
     Optional ByVal AppName As String = "IisLogHousekeeper", _
     Optional ByVal EventType As  _
     EventLogEntryType = EventLogEntryType.Information, _
     Optional ByVal LogName As String = "Application") As Boolean
    Dim objEventLog As New EventLog()
    Try
      If Not Diagnostics.EventLog.SourceExists(AppName) Then
        Diagnostics.EventLog.CreateEventSource(AppName, LogName)
      End If
      objEventLog.Source = AppName
      objEventLog.WriteEntry(Entry, EventType, EventID)
      Return True
    Catch Ex As Exception
      Return False
    End Try
    objEventLog.Dispose()
  End Function

  Public Function GetIisLogFoldersThroughADSI() As String()
    Dim sFolderList As String = ""
    Dim deIisRootObject As New DirectoryEntry("IIS://localhost")
    Dim deIisSubItems As DirectoryEntry
    Dim deIisNodeObj As DirectoryEntry
    For Each deIisSubItems in deIisRootObject.Children
      If LCase(deIisSubItems.SchemaClassName) = "iiswebservice" Then
        Dim deIisService As New DirectoryEntry("IIS://localhost/W3SVC")
        For Each deIisNodeObj in deIisService.Children
          If LCase(deIisNodeObj.SchemaClassName) = "iiswebserver" Then
            sFolderList = sFolderList & _
              deIisNodeObj.Properties("LogFileDirectory").Value.ToString() & _
              "\W3SVC" & deIisNodeObj.Name & "|"
          End If
        Next
      ElseIf LCase(deIisSubItems.SchemaClassName) = "iissmtpservice" Then
        Dim deIisService As New DirectoryEntry("IIS://localhost/SMTPSVC")
        For Each deIisNodeObj in deIisService.Children
          If LCase(deIisNodeObj.SchemaClassName) = "iissmtpserver" Then
            sFolderList = sFolderList & _
              deIisNodeObj.Properties("LogFileDirectory").Value.ToString() & _
              "\SMTPSVC" & deIisNodeObj.Name & "|"
          End If
        Next
      ElseIf LCase(deIisSubItems.SchemaClassName) = "iisftpservice" Then
        Dim deIisService As New DirectoryEntry("IIS://localhost/MSFTPSVC")
        For Each deIisNodeObj in deIisService.Children
          If LCase(deIisNodeObj.SchemaClassName) = "iisftpserver" Then
            sFolderList = sFolderList & _
              deIisNodeObj.Properties("LogFileDirectory").Value.ToString() & _
              "\MSFTPSVC" & deIisNodeObj.Name & "|"
          End If
        Next
      End If
    Next
    If Right(sFolderList, 1) = "|" Then
      sFolderList = Left(sFolderList, Len(sFolderList) - 1)
    End If
    Return Split(sFolderList, "|")
  End Function

  Public Function DeleteLogFilesRecursive(ByVal sLogPath As String, ByVal iDelAge As Integer) As String
    Dim sReturn As String = ""
    If Directory.Exists(sLogPath)
      Dim dInfo As New DirectoryInfo(sLogPath)
      Dim fInfo as FileSystemInfo
      Dim sTempString As String = ""
      For Each fInfo In dInfo.GetFileSystemInfos()
        If fInfo.Attributes = FileAttributes.Directory Then
          sReturn = sReturn & DeleteLogFilesRecursive(fInfo.FullName, iDelAge)
        Else
          If (InStr(fInfo.FullName, "ex") > 0) And (Right(fInfo.FullName, 4) = ".log") Then
            If DateDiff("d", fInfo.LastWriteTime, Now) > iDelAge Then
              Try
                sTempString = fInfo.FullName & " deleted." & VbCrLf
                If LCase(ConfigurationManager.AppSettings("SimulationMode")) = "false" Then
                  File.Delete(fInfo.FullName)
                  sReturn = sReturn & sTempString
                Else
                  sReturn = sReturn & "[Simulation mode enabled] " & sTempString
                End If
              Catch e As Exception
                sReturn = sReturn & fInfo.FullName & " could not be deleted. Error message: " & e.Message & VbCrLf
              End Try
            End If
          End If
        End If
      Next
    End If
    Return sReturn
  End Function

End Class

<System.ComponentModel.RunInstaller(True)> _
Public Class ProjectInstaller
  Inherits System.Configuration.Install.Installer

  Public Sub New()
    MyBase.New()
    InitializeComponent()
  End Sub

  Private components As System.ComponentModel.IContainer

  Private Sub InitializeComponent()
    Me.ServiceProcessInstaller1 = New ServiceProcessInstaller
    Me.ServiceInstaller1 = New ServiceInstaller
    Me.ServiceProcessInstaller1.Account = ServiceAccount.LocalSystem
    Me.ServiceProcessInstaller1.Password = Nothing
    Me.ServiceProcessInstaller1.Username = Nothing
    Me.ServiceInstaller1.ServiceName = "IisLogHousekeeper"
    Me.ServiceInstaller1.StartType = ServiceStartMode.Automatic
    Me.Installers.AddRange(New System.Configuration.Install.Installer() {Me.ServiceProcessInstaller1, Me.ServiceInstaller1})
  End Sub
  Friend WithEvents ServiceProcessInstaller1 As ServiceProcessInstaller
  Friend WithEvents ServiceInstaller1 As ServiceInstaller

End Class

<Global.Microsoft.VisualBasic.CompilerServices.DesignerGenerated()> _
Partial Class IisLogHousekeeper
  Inherits System.ServiceProcess.ServiceBase

  <System.Diagnostics.DebuggerNonUserCode()> _
  Protected Overrides Sub Dispose(ByVal disposing As Boolean)
    Try
      If disposing AndAlso components IsNot Nothing Then
        components.Dispose()
      End If
    Finally
      MyBase.Dispose(disposing)
    End Try
  End Sub

  <MTAThread()> _
  <System.Diagnostics.DebuggerNonUserCode()> _
  Shared Sub Main()
    Dim ServicesToRun() As System.ServiceProcess.ServiceBase
    ServicesToRun = New System.ServiceProcess.ServiceBase() {New IisLogHousekeeper}
    System.ServiceProcess.ServiceBase.Run(ServicesToRun)
  End Sub

  Private components As System.ComponentModel.IContainer

  <System.Diagnostics.DebuggerStepThrough()> _
  Private Sub InitializeComponent()
    components = New System.ComponentModel.Container()
    Me.ServiceName = "IisLogHousekeeper"
  End Sub

End Class 